import json
import socket

from os import environ
from typing import Optional
from http import HTTPStatus

from slack_sdk import WebhookClient
from slack_sdk.http_retry import RetryHandler, RetryState, HttpRequest, HttpResponse
from slack_sdk.http_retry.builtin_interval_calculators import BackoffRetryIntervalCalculator
from slack_sdk.http_retry.jitter import RandomJitter

from exceptions import SlackServiceException


class MyRetryHandler(RetryHandler):
    def _can_retry(
        self,
        *,
        state: RetryState,
        request: HttpRequest,
        response: Optional[HttpResponse] = None,
        error: Optional[Exception] = None,
    ) -> bool:
        # [Errno 104] Connection reset by peer
        return error is not None and isinstance(error, socket.error) and error.errno == 104


class SlackWebhookClient:
    def __init__(
        self,
        url: str,
        urn: str,
        operation: str = "",
        environment: str = "",
        org_name: str = "",
        org_key: str = "",
        org_id: str = "",
        org_uuid: str = "",
        details: dict = {},
    ):
        self.urn = urn
        self.operation = operation
        self.environment = environment
        self.org_name = org_name
        self.org_key = org_key
        self.org_id = org_id
        self.org_uuid = org_uuid
        self.details = details

        self.region = environ.get("AWS_REGION")
        self.client = WebhookClient(
            url,
            retry_handlers=[
                MyRetryHandler(
                    max_retry_count=3,
                    interval_calculator=BackoffRetryIntervalCalculator(
                        backoff_factor=1,
                        jitter=RandomJitter(),
                    ),
                )
            ],
        )

    def _get_blocks(self) -> list:
        execution_link = (
            f"https://{self.region}.console.aws.amazon.com/states/home?region={self.region}#/v2/executions/details/{self.urn}"
        )
        details_json = json.dumps(self.details, indent=2)
        return [
            {
                "type": "section",
                "text": {"type": "mrkdwn", "text": f"Execution urn `{self.urn}`"},
                "accessory": {"type": "button", "text": {"type": "plain_text", "text": "Go to execution"}, "url": execution_link},
            },
            {
                "type": "section",
                "fields": [
                    {"type": "mrkdwn", "text": f"*Operation:* `{self.operation}`"},
                    {"type": "mrkdwn", "text": f"*Environment:* `{self.environment}`"},
                    {"type": "mrkdwn", "text": f"*Org Name:* `{self.org_name}`"},
                    {"type": "mrkdwn", "text": f"*Org Key:* `{self.org_key}`"},
                    {"type": "mrkdwn", "text": f"*Org Id:* `{self.org_id}`"},
                    {"type": "mrkdwn", "text": f"*Org UUID:* `{self.org_uuid}`"},
                ],
            },
            {"type": "divider"},
            {
                "type": "section",
                "text": {
                    "type": "mrkdwn",
                    "text": f"*Details:*\n```\n{details_json}\n```",
                },
            },
        ]

    def send(self):
        response = self.client.send(blocks=self._get_blocks())

        if not (HTTPStatus.OK <= response.status_code < HTTPStatus.MULTIPLE_CHOICES):
            raise SlackServiceException(url=response.api_url, status_code=response.status_code, body=response.body)
